## load packages
library(mc2d)  # function 'rtrunc' is contained
library(MASS)  # function 'mvrnorm' is contained
library(sfsmisc)

dyn.load("./const2.so")

CountVariable <- function(Input, FlagSorting = 0, decreasing = FALSE)
  # This function finds unique members of Input and count their numbers of occurrence
  # FlagSorting: 0, do not sort; 1, sort by unique members; 2, sort by number of occurrence
  # Return values are:
  # UniqueVariable: the elements in Input.
  # NumberOccurrence: the numbers of occurrence of elements in Input.
{
  UniqueVariable = unique(Input); 
  NumOccurrence = rep(0, length(UniqueVariable));
  NumOccurrence = sapply(UniqueVariable, function(x)sum(Input == x));
  if (FlagSorting == 0)
  {
    ID = 1 : length(UniqueVariable);
  }
  if (FlagSorting == 1)
  {
    ID = order(UniqueVariable, decreasing = decreasing);
  }
  if (FlagSorting == 2)
  {
    ID = order(NumOccurrence, decreasing = decreasing);
  } 
  Result = list(UniqueVariable = UniqueVariable[ID], NumberOccurrence = NumOccurrence[ID]);
  Result;
}

################################################################
## 6. functions for initialization
################################################################

initial.values <- function(Nnode, Nsample, y)
{
	e.ini<-matrix(0,Nnode,Nsample)
	for (i in 1:Nnode){
		e.ini[i,which(y[i,]<quantile(y[i,],0.10))]=-1;  e.ini[i,which(y[i,]>quantile(y[i,],0.90))]=1;
	}
	z.ini<-matrix(0,Nnode,Nsample)
	z.ini[which(e.ini==-1)]=0
	z.ini[-which(e.ini==-1)]=1
	
	p.ini=rep(0,Nnode)
	for (i in 1:Nnode){
    p.ini[i]<-ifelse(sum(z.ini[i,]==1)!=0,sum(e.ini[i,]==0)/sum(z.ini[i,]==1),0.5)}
    
	beta.ini = matrix(0,Nnode,Nnode)
	diag(beta.ini) = rnorm(Nnode,0,1)
	
	mu.ini=rep(0,Nnode); sigma.ini=rep(0,Nnode); kminus.ini=rep(0,Nnode); kplus.ini=rep(0,Nnode);
	for (i in 1:Nnode){
		mu.ini[i]=mean(y[i,]);
		sigma.ini[i]=sd(y[i,])*0.7;
		kminus.ini[i]=ifelse(min(y[i,])<0,abs(min(y[i,])-0.5),max(y[i,])+0.5)
	kplus.ini[i]=ifelse(max(y[i,])>0, max(y[i,])+0.5, abs(min(y[i,])-0.5)) }
	
	return(list(e.ini=e.ini, z.ini=z.ini, p.ini=p.ini, beta.ini=beta.ini,
				
				sigma.ini=sigma.ini, kminus.ini=kminus.ini, kplus.ini=kplus.ini,
				
				mu.ini=mu.ini))
	
}


NetworkAnalysis <- function(Input) 
{

  start.time = proc.time();
  Nnode = dim(Input$y)[1];
  Nsample = dim(Input$y)[2];
  
###################################################################
## 9. Main excution of simulation
###################################################################
N = Input$Rep  # number of simulations
burn.in = 6000
  
set.seed(1)  # ZYT changed.

graph.simu = array(0,c(Nnode,Nnode,N))  # each row is a simulation path

beta.simu = array(0,c(Nnode,Nnode,N))   # each row is a simulation path

##(2) compute initial values

init=initial.values(Nnode = Nnode, Nsample = Nsample, y = Input$y)

## (3) gibbs sampling

yvector <- as.double(Input$y)

# Gibbs is the main C function to run MCMC. The C function also calls some basic R functions.  
tmp <- .C("Gibbs", yvector = yvector, betavector = as.double(beta.simu), graphvector = as.integer(graph.simu), evector = as.integer(init$e.ini), zvector = as.integer(init$z.ini), 
p = as.double(init$p.ini), mu = as.double(init$mu.ini), sigma = as.double(init$sigma.ini), kminu = as.double(init$kminus.ini),
kplus = as.double(init$kplus.ini), inibetavector = as.double(init$beta.ini), inigraphvector = as.integer(Input$graph.prior), graphpriorvector = as.integer(Input$graph.prior) , Nnode = as.integer(Nnode), Nsample = as.integer(Nsample), pho = as.double(Input$pho), N = as.integer(N))

beta.simu = array(tmp$betavector, c(Nnode, Nnode, N))
graph.simu = array(tmp$graphvector, c(Nnode, Nnode, N))
  
## (6) output
lag = 10
NumPos = (N - burn.in)/lag

post.mean.beta=matrix(NA,Nnode,Nnode) ##the posterior mean of edge coefficient beta
post.sd.beta=matrix(NA,Nnode,Nnode) ##the posterior sd of edge coefficient beta  
prob.edge = matrix(NA,Nnode,Nnode) ## the posterior probability of each edge
for (i in 1:Nnode){
for (j in 1:Nnode){
post.mean.beta[i,j]=mean(beta.simu[i,j,seq(burn.in+lag,N,lag)])
post.sd.beta[i,j]=sd(beta.simu[i,j,seq(burn.in+lag,N,lag)])
prob.edge[i,j]<-sum(graph.simu[i,j,seq(burn.in+lag,N,lag)]==1)/NumPos
}
}

##store the FDR for each edge
FdrProb = prob.edge[upper.tri(prob.edge)]
ids <- order(FdrProb,decreasing=T)
fdr <- cumsum(sort(1-FdrProb))/(1:(Nnode*(Nnode-1)/2))
fdrorder = rep(NA,(Nnode*(Nnode-1)/2))
fdrorder[ids] <- fdr
FDR = matrix(0, Nnode, Nnode)
FDR[upper.tri(FDR)] <- fdrorder
FDR[lower.tri(FDR)] <- t(FDR)[lower.tri(FDR)]

##store the posterior graphs
graphinte.simu = rep(0,NumPos)
digit <- 2^((Nnode*(Nnode-1)/2-1):0)
for (i in 1:NumPos)
{
j <- burn.in + i*lag
graph.tmp <- graph.simu[,,j]
graphinte.simu[i] <- graph.tmp[upper.tri(graph.tmp)]%*%digit
}

graphinte.simu = CountVariable(Input = graphinte.simu, FlagSorting = 2, decreasing = TRUE);
graphMatrixList = vector("list", length = min(length(graphinte.simu$UniqueVariable), 3));
for (GraphIndex in 1:min(length(graphinte.simu$UniqueVariable), 3))
{
  #restore the graph from graphinte.simu
  #Assume the graph number is a
 
  if (graphinte.simu$UniqueVariable[GraphIndex] == 0)
  {
    graph = diag(Nnode);
  }  
  else
  {
    b = digitsBase(graphinte.simu$UniqueVariable[GraphIndex]);
    NumLen = (Nnode-1)*Nnode/2;
    if (length(b)<NumLen) { b<- c(rep(0, NumLen-length(b)), b);}
    graph<-matrix(1, Nnode, Nnode);
    graph[upper.tri(graph)] <- b;
    graph[lower.tri(graph)] <- t(graph)[lower.tri(graph)];  
  } 
  graphMatrixList[[GraphIndex]] = graph;
}
names(graphMatrixList) = paste("Frequency ", round(graphinte.simu$NumberOccurrence[1:min(3, length(graphinte.simu$NumberOccurrence))]/NumPos, digits = 4), sep = "");
  
end.time = proc.time();
Time = end.time - start.time;
Result = list(post.mean.beta = post.mean.beta, post.sd.beta = post.sd.beta, prob.edge = prob.edge, FDR = FDR, graphMatrixList = graphMatrixList);
  
}
